#include <stdint.h>
#include <stdlib.h>
#include <desc_tables.h>

extern void GDTFlush(uint32);
extern void IDTFlush(uint32);

void initGDT(void);
void initIDT(void);

void GDTSetGate(sint32,uint32,uint32,uint8,uint8);
void IDTSetGate(uint8,uint32,uint16,uint8);

GDTEntry gdtEntries[5];
GDTPtr gdtPtr;
IDTEntry idtEntries[256];
IDTPtr idtPtr;

void initDT(void)
{
	initGDT();
	initIDT();
}

void initGDT(void)
{
	gdtPtr.limit = (sizeof(GDTEntry) * 5 ) - 1;
	gdtPtr.base = (uint32)&gdtEntries;
	
	GDTSetGate(0,0,0,0,0);
	GDTSetGate(1,0,0xFFFFFFFF,0x9A,0xCF);
	GDTSetGate(2,0,0xFFFFFFFF,0x92,0xCF);
	GDTSetGate(3,0,0xFFFFFFFF,0xFA,0xCF);
	GDTSetGate(4,0,0xFFFFFFFF,0xF2,0xCF);
	
	GDTFlush((uint32)&gdtPtr);
}

void initIDT(void)
{
	idtPtr.limit = sizeof(IDTEntry) * 256 - 1;
	idtPtr.base = (uint32)&idtEntries;
	
	memset((uint8 *)&idtEntries,0,sizeof(IDTEntry) * 256);
	IDTSetGate(0,(uint32)isr0,0x08,0x8E);
	IDTSetGate(1,(uint32)isr1,0x08,0x8E);
	IDTSetGate(2,(uint32)isr2,0x08,0x8E);
	IDTSetGate(3,(uint32)isr3,0x08,0x8E);
	IDTSetGate(4,(uint32)isr4,0x08,0x8E);
	IDTSetGate(5,(uint32)isr5,0x08,0x8E);
	IDTSetGate(6,(uint32)isr6,0x08,0x8E);
	IDTSetGate(7,(uint32)isr7,0x08,0x8E);
	IDTSetGate(8,(uint32)isr8,0x08,0x8E);
	IDTSetGate(9,(uint32)isr9,0x08,0x8E);
	IDTSetGate(10,(uint32)isr10,0x08,0x8E);
	IDTSetGate(11,(uint32)isr11,0x08,0x8E);
	IDTSetGate(12,(uint32)isr12,0x08,0x8E);
	IDTSetGate(13,(uint32)isr13,0x08,0x8E);
	IDTSetGate(14,(uint32)isr14,0x08,0x8E);
	IDTSetGate(15,(uint32)isr15,0x08,0x8E);
	IDTSetGate(16,(uint32)isr16,0x08,0x8E);
	IDTSetGate(17,(uint32)isr17,0x08,0x8E);
	IDTSetGate(18,(uint32)isr18,0x08,0x8E);
	IDTSetGate(19,(uint32)isr19,0x08,0x8E);
	IDTSetGate(20,(uint32)isr20,0x08,0x8E);
	IDTSetGate(21,(uint32)isr21,0x08,0x8E);
	IDTSetGate(22,(uint32)isr22,0x08,0x8E);
	IDTSetGate(23,(uint32)isr23,0x08,0x8E);
	IDTSetGate(24,(uint32)isr24,0x08,0x8E);
	IDTSetGate(25,(uint32)isr25,0x08,0x8E);
	IDTSetGate(26,(uint32)isr26,0x08,0x8E);
	IDTSetGate(27,(uint32)isr27,0x08,0x8E);
	IDTSetGate(28,(uint32)isr28,0x08,0x8E);
	IDTSetGate(29,(uint32)isr29,0x08,0x8E);
	IDTSetGate(30,(uint32)isr30,0x08,0x8E);
	IDTSetGate(31,(uint32)isr31,0x08,0x8E);
}

void GDTSetGate(sint32 num,uint32 base, uint32 limit, uint8 access, uint8 gran)
{
	gdtEntries[num].baseLow = (base & 0xFFFF);
	gdtEntries[num].baseMiddle = (base >> 16) & 0xFF;
	gdtEntries[num].baseHigh = (base >> 24) & 0xFF;
	gdtEntries[num].limitLow = (limit & 0xFFFF);
	gdtEntries[num].granularity = (limit >> 16) & 0x0F;
	gdtEntries[num].granularity |= gran & 0xF0;
	gdtEntries[num].access = access;
}

void IDTSetGate(uint8 num, uint32 base, uint16 sel, uint8 flags)
{
	idtEntries[num].baseLow = base & 0xFFFF;
	idtEntries[num].baseHigh = (base >> 16) & 0xFFFF;
	idtEntries[num].sel = sel;
	idtEntries[num].always0 = 0;
	idtEntries[num].flags = flags;
}